@@ -1,7 +1,7 @@
 # $Id: ODBC.pm,v 1.12 1998/08/14 19:29:50 timbo Exp $
 #
 # Copyright (c) 1994,1995,1996,1998  Tim Bunce
-# portions Copyright (c) 1997,1998  Jeff Urlwin
+# portions Copyright (c) 1997,1998,1999,2000,2001,2002  Jeff Urlwin
 # portions Copyright (c) 1997  Thomas K. Wenrich
 #
 # You may distribute under the terms of either the GNU General Public
@@ -9,7 +9,7 @@
 
 require 5.004;
 
-$DBD::ODBC::VERSION = '0.43';
+$DBD::ODBC::VERSION = '1.00';
 
 {
     package DBD::ODBC;
@@ -44,12 +44,13 @@ $DBD::ODBC::VERSION = '0.43';
 	    'Err'    => \$DBD::ODBC::err,
 	    'Errstr' => \$DBD::ODBC::errstr,
 	    'State' => \$DBD::ODBC::sqlstate,
-	    'Attribution' => 'ODBC DBD by Tim Bunce',
+	    'Attribution' => 'ODBC DBD by Jeff Urlwin',
 	    });
 
 	$drh;
     }
 
+    sub CLONE { undef $drh }
     1;
 }
 
@@ -417,9 +418,14 @@ See L<DBI> for more information.
 
 =head1 DESCRIPTION
 
-=head2 Recent Updates
+=head2 Notes:
 
 =over 4
+=item B<Change log/recent updates now in DBD::ODBC::Changes.pm>
+
+ Please note that the change log has been moved to DBD::ODBC::Changes.pm
+ To easily access this documentation, use perldoc DBD::ODBC::Changes
+ 
 =item B<An Important note about the tests!>
 
  Please note that some tests may fail or report they are
@@ -433,268 +439,89 @@ See L<DBI> for more information.
    
  Also note that some tests may be skipped, such as
  t/09multi.t, if your driver doesn't seem to support
- returning multiple result sets.
+ returning multiple result sets.  This is normal.
 
-=item B<DBD::ODBC 0.43>
+=item B<Private DBD::ODBC Attributes>
+=item odbc_more_results (applies to statement handle only!)
 
-Fix for FoxPro drivers -- HURRAY!!!!!!!!!  Bad call to SQLDescribeCol where the FoxPro driver
-seems to clear out the buffer passed and the size of the buffer indicated in the parameters
-was larger than the real buffer...need better fix long term, but fixed for now by allocating
-extra space.
+Use this attribute to determine if there are more result sets
+available.  SQL Server supports this feature.  Use this as follows:
+
+do {
+   my @row;
+   while (@row = $sth->fetchrow_array()) {
+      # do stuff here
+   }
+} while ($sth->{odbc_more_results});
+
+Note that with multiple result sets and output parameters (i.e. using
+bind_param_inout, don't expect output parameters to be bound until ALL
+result sets have been retrieved.
+
+=item odbc_ignore_named_placeholders
+
+Use this if you have special needs (such as Oracle triggers, etc) where
+:new or :name mean something special and are not just place holder names
+You I<must> then use ? for binding parameters.  Example:
+ $dbh->{odbc_ignore_named_placeholders} = 1;
+ $dbh->do("create trigger foo as if :new.x <> :old.x then ... etc");
+
+Without this, DBD::ODBC will think :new and :old are placeholders for binding
+and get confused.
  
-Added support for column_info (maps to function columns, previously provided)
- 
-Fix for binding undef value generated by a dereferencing an empty hash value.
-I.e in the hash, there is no key value "1", but $hash{"1"} is passed to binding parameters.
+=item odbc_default_bind_type
 
-A fix to make all bound columns word aligned.  This solved a problem on solaris when binding
-timestamp values after a varchar value.  This patch forces all columns to start on word
-boundaries (sizeof int boundaries, actually).  This may have a positive speed impact for
-other systems.  Thanks to Joe Slagel for pointing this out and providing a patch for the fix!
- 
-=item B<DBD::ODBC 0.42>
+This value defaults to 0.  Older versions of DBD::ODBC assumed that the binding
+type was 12 (SQL_VARCHAR).  Newer versions default to 0, which means that
+DBD::ODBC will attempt to query the driver via SQLDescribeParam to determine
+the correct type.  If the driver doesn't support SQLDescribeParam, then DBD::ODBC
+falls back to using SQL_VARCHAR as the default, unless overridden by bind_param()
 
-A patch to the tests to get past any potential problems with
-a user not defining the DBI_DSN environment variable.  Hopefully
-this will allow ActiveState to keep up with DBD::ODBC in
-their automated builds.  Thanks to Jan Dubois for helping
-me determine what the issue(s) may be when testing.
-Now skips all tests except the dll/so load if DBI_DSN is
-not defined.  Also, issues a skip on a known Oracle ODBC
-test.
- 
-A patch to fix ping() when a S1000 was returned.  I don't know how this
-one is going to go, but it's worth a shot.  Now calls SQLTables with unlikely
-parameters.  If SQLTables() succeeds and returns a result set, ping() will
-now return true.  Thanks to Tim Bunce for the suggestion.
- 
-=item B<DBD::ODBC 0.41>
+=item odbc_force_rebind
 
-A patch to handle SQLDescribeParam failing in some
-circumstances.  I believe this is a bug in the ODBC
-driver, but this happens with SQL Server and INNER
-JOIN syntax queries, but not when where a.i = b.i.
-The behavior is now to revert to assume SQL_VARCHAR
-if SQLDescribeParam fails for whatever reason.					     
+This is to handle special cases, especially when using multiple result sets.
+Set this before execute to "force" DBD::ODBC to re-obtain the result set's
+number of columns and column types for each execute.  Especially useful for
+calling stored procedures which may return different result sets each
+execute.  The only performance penalty is during execute(), but I didn't
+want to incur that penalty for all circumstances.  It is probably fairly
+rare that this occurs.  This attribute will be automatically set when
+multiple result sets are triggered.  Most people shouldn't have to worry
+about this.
 
-Added warning during tests when the Oracle ODBC driver
-is detected and it is a known error.  I didn't want
-to make it pass the test silently, but I did want to print
-a message indicating it was expected.
+=item odbc_async_exec
+
+Allow asynchronous execution of queries.  Right now, this causes a spin-loop
+(with a small "sleep") until the sql is complete.  This is useful, however,
+if you want the error handling and asynchronous messages (see the err_handler)
+below.  See t/20SQLServer.t for an example of this.
+
+=item odbc_exec_direct
+
+Force DBD::ODBC to use SQLExecDirect instead of SQLPrepare() then SQLExecute.
+There are drivers that only support SQLExecDirect and the DBD::ODBC
+do() override doesn't allow returning result sets.  Therefore, the
+way to do this now is to set the attributed odbc_exec_direct.
+There are currently two ways to get this:
+	$dbh->prepare($sql, { odbc_execdirect => 1}); 
+ and
+	$dbh->{odbc_execdirect} = 1;
+ When $dbh->prepare() is called with the attribute "ExecDirect" set to a non-zero value 
+ dbd_st_prepare do NOT call SQLPrepare, but set the sth flag odbc_exec_direct to 1.
+ 
+=item odbc_err_handler
+
+Allow errors to be handled by the application.  A call-back function supplied
+by the application to handle or ignore messages.  If the error handler returns
+0, the error is ignored, otherwise the error is passed through the normal
+DBI error handling structure(s).
+ 
+=item odbc_SQL_ROWSET_SIZE
+
+Here is the information from the original patch, however, I've learned
+since from other sources that this could/has caused SQL Server to "lock up".
+Please use at your own risk!
    
-=item B<DBD::ODBC 0.40>
-
-Two minor patches for building.  One for Cygwin support
-and another to handle the case where both iODBC and
-unixODBC libraries are installed.  The preference is
-now given to unixODBC...  
-
-Fixed problem in connect code introduced in 0.39 where
-new code would only be executed if tracing was on and
-> 3...
-
-NEW: Changed default bind type to 0, to let DBD::ODBC
-try to detect the correct bind type.  This is a
-potentially significant change.  Please let me know
-if there is a problem.  The best way is to post a message
-to the dbi-users@perl.org mailing list.
-
-=item B<DBD::ODBC 0.39>
-
- Removing iodbcsrc directory as newer/better versions
- can be found on the web at www.iodbc.org and
- www.unixodbc.org.  On Linux, I currently build
- with unixODBC verson 2.2.0.
- 
- Added patch for handling setting the ODBC environment
- during the connect, thanks to Steffen Goeldner
- 
- The attached patch makes it possible to choose the
- ODBC version. E.g.:
- 
-   my $dbh = DBI->connect( ..., { odbc_version => 3 } )
- 
- directs the driver to exhibit ODBC 3.x behavior.
-
- Fix to SQLColAttributes thanks to Nicolas DeRico
-
- Changes to the connect sequence.  If SQLDriverConnect
- is supported and fails, SQLConnect will be called
- I<unless> (this is the new part) the length of the
- DSN is > SQL_MAX_DSN_LENGTH or the DSN begins with
- DSN=
-
- New test in mytest which demonstrates long binary
- types.  If you are having trouble inserting images
- into your database, please check here.
-=item B<DBD::ODBC 0.38>
-
- Fixed do function (again) thanks to work by Martin Evans.
- 
-=item B<DBD::ODBC 0.37>
-
- Patches for get_info where return type is string.  Patches
- thanks to Steffen Goldner.  Thanks Steffen!
-
- Patched get_info to NOT attempt to get data for SQL_DRIVER_HSTMT
- and SQL_DRIVER_HDESC as they expect data in and have limited value
- (IMHO).
-
-=item B<DBD::ODBC 0.37>
-
- Further fixed build for ODBC 2.x drivers.  The new SQLExecDirect
- code had SQLAllocHandle which is a 3.x function, not a 2.x function.
- Sigh.  I should have caught that the first time.  Signed, the Mad-and-
- not-thorough-enough-patcher.
-
- Additionally, a random core dump occurred in the tests, based upon the
- new SQLExecDirect code.  This has been fixed.
- 
- 
-=item B<DBD::ODBC 0.36>
-
- Fixed build for ODBC 2.x drivers.  The new SQLExecDirect code
- had SQLFreeHandle which is a 3.x function, not a 2.x function.
- 
-=item B<DBD::ODBC 0.35>
-
- Fixed (finally) multiple result sets with differing
- numbers of columns.  The final fix was to call
- SQLFreeStmt(SQL_UNBIND) before repreparing
- the statement for the next query.
-
- Added more to the multi-statement tests to ensure
- the data retrieved was what was expected.
-
- Now, DBD::ODBC overrides DBI's do to call SQLExecDirect
- for simple statements (those without parameters).
- Please advise if you run into problems.  Hopefully,
- this will provide some small speed improvement for
- simple "do" statements.  You can also call
- $dbh->func($stmt, ExecDirect).  I'm not sure this has
- great value unless you need to ensure SQLExecDirect
- is being called.  Patches thanks to Merijn Broeren.
- Thanks Merijn!
-   
-=item B<DBD::ODBC 0.34>
- 
- Further revamped tests to attempt to determine if SQLDescribeParam
- will work to handle the binding types.  The t/08bind.t attempts
- to determine if SQLDescribeParam is supported.  note that Oracle's
- ODBC driver under NT doesn't work correctly when binding dates
- using the ODBC date formatting {d } or {ts }.  So, test #3 will
- fail in t/08bind.t
-
- New support for primary_key_info thanks to patches by Martin Evans.
- New support for catalog, schema, table and table_type in table_info
- thanks to Martin Evans.  Thanks Martin for your work and your
- continuing testing, suggestions and general support!
-
- Support for upcoming dbi get_info.
- 
-=item B<DBD::ODBC 0.33_3>
-
- Revamped tests to include tests for multiple result sets.
- The tests are ODBC driver platform specific and will be skipped
- for drivers which do not support multiple result sets.
- 
-=item B<DBD::ODBC 0.33_2>
-
- Finally tested new binding techniques with SQL Server 2000,
- but there is a nice little bug in their MDAC and ODBC
- drivers according to the knowledge base article # Q273813, titled
-
-   "FIX: "Incorrect Syntax near the Keyword 'by' "
-   Error Message with Column Names of "C", "CA" or "CAS" (Q273813)
-
- DBD::ODBC now does not name any of the columns A, B, C, or D
- they are now COL_A, COL_B, COL_C, COL_D.
-
- *** NOTE: *** I AM STRONGLY CONSIDERING MAKING THE NEW
- BINDING the default for future versions.  I do not believe
- it will break much existing code (if any) as anyone binding
- to non VARCHAR (without the ODBC driver doing a good conversion
- from the VARCHAR) will have a problem.  It may be subtle, however,
- since much code will work, but say, binding dates may not with
- some drivers.
-   
- Please comment soon...
-   
-=item B<DBD::ODBC 0.33_1>
-
-*** WARNING: ***
- 
- Changes to the binding code to allow the use of SQLDescribeParam
- to determine if the type of column being bound.  This is
- experimental and activated by setting
- 
-  $dbh->{odbc_default_bind_type} = 0; # before creating the query...
-
- Currently the default value of odbc_default_bind_type = SQL_VARCHAR
- which mimicks the current behavior.  If you set
- odbc_default_bind_type to 0, then SQLDescribeParam will be
- called to determine the columen type.  Not ALL databases
- handle this correctly.  For example, Oracle returns
- SQL_VARCHAR for all types and attempts to convert to the
- correct type for us.  However, if you use the ODBC escaped
- date/time format such as: {ts '1998-05-13 00:01:00'} then
- Oracle complains.  If you bind this with a SQL_TIMESTAMP type,
- however, Oracle's ODBC driver will parse the time/date correctly.
- Use at your own risk!
-
- Fix to dbdimp.c to allow quoted identifiers to begin/end
- with either " or '.
- The following will not be treated as if they have a bind placeholder:
-   "isEstimated?"
-   '01-JAN-1987 00:00:00'
-   'Does anyone insert a ?'
-
-				    
-=item B<DBD::ODBC 0.32>
-
- More SAP patches to Makfile.PL to eliminate the call to Data Sources
-
- A patch to the test (for SAP and potentially others), to allow
- fallback to SQL_TYPE_DATE in the tests
- 
-=item B<DBD::ODBC 0.31>
-
- Added SAP patches to build directly against SAP driver instead of
- driver manager thanks to Flemming Frandsen (thanks!)
-
- Added support to fix ping for Oracle8.  May break other databases,
- so please report this as soon as possible.  The downside is that
- we need to actually execute the dummy query.
- 
-
-=item B<DBD::ODBC 0.30>
-
- Added ping patch for Solid courtesy of Marko Asplund
-
- Updated disconnect to rollback if autocommit is not on.
- This should silence some errors when disconnecting.
-
- Updated SQL_ROWSET_SIZE attribute.  Needed to force it to
- odbc_SQL_ROWSET_SIZE to obey the DBI rules.
-
- Added odbc_SQL_DRIVER_ODBC_VER, which obtains the version of
- the Driver upon connect.  This internal capture of the version is
- a read-only attributed and is used during array binding of parameters.
- 
- Added odbc_ignore_named_placeholders attribute to facilicate
- creating triggers within SAPDB and Oracle, to name two. The
- syntax in these DBs is to allow use of :old and :new to
- access column values before and after updates.  Example:
-
- $dbh->{odbc_ignore_named_placeholders} = 1; # set it for all future statements
-					  # ignores :foo, :new, etc, but not :1 or ?
- $dbh->do("create or replace etc :new.D = sysdate etc");
- 
-
-=item B<DBD::ODBC 0.29>
-
- Cygwin patches from Neil Lunn (untested by me).  Thanks Neil!
- 
 SQL_ROWSET_SIZE attribute patch from Andrew Brown 
 > There are only 2 additional lines allowing for the setting of
 > SQL_ROWSET_SIZE as db handle option.
@@ -707,7 +534,7 @@ SQL_ROWSET_SIZE attribute patch from Andrew Brown
 > selects are possible.
 >
 > The code change allows for:
-> $dbh->{odbc_SQL_ROWSET_SIZE} = 2;    # Any value > 1
+> $dbh->{SQL_ROWSET_SIZE} = 2;    # Any value > 1
 >
 > For this very purpose.
 >
@@ -718,228 +545,20 @@ SQL_ROWSET_SIZE attribute patch from Andrew Brown
 > Andrew
 >
 
-VMS and other patches from Martin Evans (thanks!)
+=item SQL_DRIVER_ODBC_VER
 
-[1] a fix for Makefile.PL to build DBD::ODBC on OpenVMS.
-
-[2] fix trace message coredumping after SQLDriverConnect
-
-[3] fix call to SQLCancel which fails to pass the statement handle properly.
-
-[4] consume diagnostics after SQLDriverConnect/SQLConnect call or they remain
-    until the next error occurs and it then looks confusing (this is due to
-    ODBC spec for SQLError). e.g. test 02simple returns a data truncated error
-    only now instead of all the informational diags that are left from the
-    connect call, like the "database changed", "language changed" messages you
-    get from MS SQL Server.
-
-Replaced C++ style comments with C style to support more platforms more easily.
-
-Fixed bug which use the single quote (') instead of a double quote (") for "literal" column names.  This
-   helped when having a colon (:) in the column name.
-
-Fixed bug which would cause DBD::ODBC to core-dump (crash) if DBI tracing level was greater than 3.
-
-Fixed problem where ODBC.pm would have "use of uninitialized variable" if calling DBI's type_info.
-
-Fixed problem where ODBC.xs *may* have an overrun when calling SQLDataSources. 
-
-Fixed problem with DBI 1.14, where fprintf was being called instead of PerlIO_printf for debug information
-
-Fixed problem building with unixODBC per patch from Nick Gorham   
-
-Added ability to bind_param_inout() via patches from Jeremy Cooper.  Haven't figured out a good, non-db specific
-   way to test.  My current test platform attempts to determine the connected database type via
-   ugly hacks and will test, if it thinks it can.  Feel free to patch and send me something...Also, my
-   current Oracle ODBC driver fails miserably and dies.
-
-Updated t/02simple.t to not print an error, when there is not one.
-   
-=item B<DBD::ODBC 0.28>
-
-Added support for SQLSpecialColumns thanks to patch provided by Martin J. Evans [martin@easysoft.com]
-
-Fixed bug introduced in 0.26 which was introduced of SQLMoreResults was not supported by the driver.
-
-=item B<DBD::ODBC 0.27>
-
-Examined patch for ping method to repair problem reported by Chris Bezil.  Thanks Chris!
-
-Added simple test for ping method working which should identify this in the future.
-
-=item B<DBD::ODBC 0.26>
-
-Put in patch for returning only positive rowcounts from dbd_st_execute.  The original patch
-was submitted by Jon Smirl and put back in by David Good.  Reasoning seems sound, so I put it
-back in.  However, any databases that return negative rowcounts for specific reasons,
-will no longer do so.
-
-Put in David Good's patch for multiple result sets.  Thanks David!  See mytest\moreresults.pl for
-an example of usage.
-
-Added readme.txt in iodbcsrc explaining an issue there with iODBC 2.50.3 and C<data_sources>.
-
-Put in rudimentary cancel support via SQLCancel.  Call $sth->cencel to utilize.  However, it is largely
-untested by me, as I do not have a good sample for this yet.  It may come in handy with threaded
-perl, someday or it may work in a signal handler.
-   
-=item B<DBD::ODBC 0.25>
-
-Added conditional compilation for SQL_WVARCHAR and SQL_WLONGVARCHAR.  If they
-are not defined by your driver manager, they will not be compiled in to the code.
-If you would like to support these types on some platforms, you may be able to
- #define SQL_WVARCHAR (-9)
- #define SQL_WLONGVARCHAR (-10)
-
-Added more long tests with binding in t\09bind.t.  Note use of bind_param!
+This, while available via get_info() is captured here.  I may get rid of this
+as I only used it for debugging purposes.  
  
-=item B<DBD::ODBC 0.24>
+=item odbc_version
 
-Fixed Test #13 in 02simple.t.  Would fail, improperly, if there was only one data source defined.
+This was added prior to the move to ODBC 3.x to allow the caller to "force" ODBC 3.0
+compatibility.  It's probably not as useful now, but it allowed get_info and get_type_info
+to return correct/updated information that ODBC 2.x didn't permit/provide.
+   
+=item B<Private DBD::ODBC Functions>
 
-Fixed (hopefully) SQL Server 7 and ntext type "Out of Memory!" errors via patch from Thomas Lowery.  Thanks Thomas!
-
-Added more support for Solid to handle the fact that it does not support data_sources nor SQLDriverConnect.
- Patch supplied by Samuli Karkkainen [skarkkai@woods.iki.fi].  Thanks!  It's untested by me, however.
-
-Added some information from Adam Curtin about a bug in iodbc 2.50.3's data_sources.  See
-   iodbcsrc\readme.txt.
-
-Added information in this pod from Stephen Arehart regarding DSNLess connections.
-
-Added fix for sp_prepare/sp_execute bug reported by Paul G. Weiss.
-
-Added some code for handling a hint on disconnect where the user gets an error for not committing.
- 
-=item B<DBD::ODBC 0.22>
-
-Fixed for threaded perl builds.  Note that this was tested only on Win32, with no threads in use and using DBI 1.13.
-Note, for ActiveState/PERL_OBJECT builds, DBI 1.13_01 is required as of 9/8/99.  
-If you are using ActiveState's perl, this can be installed by using PPM.
-
-
-=item B<DBD::ODBC 0.21>
-
-Thanks to all who provided patches!
-
-Added ability to connect to an ODBC source without prior creation of DSN.  See mytest/contest.pl for example with MS Access.
-(Also note that you will need documentation for your ODBC driver -- which, sadly, can be difficult to find).
-
-Fixed case sensitivity in tests.
-
-Hopefully fixed test #4 in t/09bind.t.  Updated it to insert the date column and updated it to find the right
-type of the column.  However, it doesn't seem to work on my Linux test machine, using the OpenLink drivers 
-with MS-SQL Server (6.5).  It complains about binding the date time.  The same test works under Win32 with 
-SQL Server 6.5, Oracle 8.0.3 and MS Access 97 ODBC drivers.  Hmmph.
-
-Fixed some binary type issues (patches from Jon Smirl)
-
-Added SQLStatistics, SQLForeignKeys, SQLPrimaryKeys (patches from Jon Smirl)
-Thanks (again), Jon, for providing the build_results function to help reduce duplicate code!
-
-Worked on LongTruncOk for Openlink drivers.
-
-Note: those trying to bind variables need to remember that you should use the following syntax:
-
-	use DBI;
-	...
-	$sth->bind_param(1, $str, DBI::SQL_LONGVARCHAR);
-
-Added support for unixodbc (per Nick Gorham)
-Added support for OpenLinks udbc (per Patrick van Kleef)
-Added Support for esodbc (per Martin Evans)
-Added Support for Easysoft (per Bob Kline)
-
-Changed table_info to produce a list of views, too.
-Fixed bug in SQLColumns call.
-Fixed blob handling via patches from Jochen Wiedmann.
-Added data_sources capability via snarfing code from DBD::Adabas (Jochen Wiedmann)
-
-=item B<DBD::ODBC 0.20>
-
-SQLColAttributes fixes for SQL Server and MySQL. Fixed tables method
-by renaming to new table_info method. Added new tyoe_info_all method.
-Improved Makefile.PL support for Adabase.
-
-=item B<DBD::ODBC 0.19>
-
-Added iODBC source code to distribution.Fall-back to using iODBC header
-files in some cases.
-
-=item B<DBD::ODBC 0.18>
-
-Enhancements to build process. Better handling of errors in
-error handling code.
-
-=item B<DBD::ODBC 0.17>
-
-This release is mostly due to the good work of Jeff Urlwin.
-My eternal thanks to you Jeff.
-
-Fixed "SQLNumResultCols err" on joins and 'order by' with some
-drivers (see Microsoft Knowledge Base article #Q124899).
-Thanks to Paul O'Fallon for that one.
-
-Added more (probably incomplete) support for unix ODBC in Makefile.PL
-
-Increased default SQL_COLUMN_DISPLAY_SIZE and SQL_COLUMN_LENGTH to 2000
-for drivers that don't provide a way to query them dynamically. Was 100!
-
-When fetch reaches the end-of-data it automatically frees the internal
-ODBC statement handle and marks the DBI statement handle as inactive
-(thus an explicit 'finish' is *not* required).
-
-Also:
-
-  LongTruncOk for Oracle ODBC (where fbh->datalen < 0)
-  Added tracing into SQLBindParameter (help diagnose oracle odbc bug)
-  Fixed/worked around bug/result from Latest Oracle ODBC driver where in
-     SQLColAttribute cbInfoValue was changed to 0 to indicate fDesc had a value
-  Added work around for compiling w/ActiveState PRK (PERL_OBJECT)
-  Updated tests to include date insert and type
-  Added more "backup" SQL_xxx types for tests                                  
-  Updated bind test to test binding select
-  NOTE: bind insert fails on Paradox driver (don't know why)
-
-Added support for: (see notes below)
-
-  SQLGetInfo       via $dbh->func(xxx, GetInfo)
-  SQLGetTypeInfo   via $dbh->func(xxx, GetTypeInfo)
-  SQLDescribeCol   via $sth->func(colno, DescribeCol)
-  SQLColAttributes via $sth->func(xxx, colno, ColAttributes)
-  SQLGetFunctions  via $dbh->func(xxx, GetFunctions)
-  SQLColumns       via $dbh->func(catalog, schema, table, column, 'columns')
-
-Fixed $DBI::err to reflect the real ODBC error code
-which is a 5 char code, not necessarily numeric.
-
-Fixed fetches when LongTruncOk == 1.
-
-Updated tests to pass more often (hopefully 100% <G>)
-
-Updated tests to test long reading, inserting and the LongTruncOk attribute.
-
-Updated tests to be less driver specific.  
-
-They now rely upon SQLGetTypeInfo I<heavily> in order to create the tables.
-The test use this function to "ask" the driver for the name of the SQL type
-to correctly create long, varchar, etc types.  For example, in Oracle the
-SQL_VARCHAR type is VARCHAR2, while MS Access uses TEXT for the SQL Name.  
-Again, in Oracle the SQL_LONGVARCHAR is LONG, while in Access it's MEMO.
-The tests currently handle this correctly (at least with Access and Oracle,
-MS SQL server will be tested also).
-
-=head2 Private functions for ODBC API access
-
-It is anticipated that at least some of the functions currently
-implemented via the C<func> interface be "moved" into a more formal,
-DBI specification.  This will be when the DBI specification
-supports/formalizes the meta-data to implement.  Most of these
-functions are to obtain more information from the driver and the data
-source.
-
-
-=item GetInfo
+=item GetInfo (superceded by get_info(), the DBI standard)
 
 This function maps to the ODBC SQLGetInfo call.  This is a Level 1 ODBC
 function.  An example of this is:
@@ -947,9 +566,10 @@ function.  An example of this is:
   $value = $dbh->func(6, GetInfo);
 
 This function returns a scalar value, which can be a numeric or string value.  
-This depends upon the argument passed to GetInfo. 
+This depends upon the argument passed to GetInfo.
 
-=item SQLGetTypeInfo
+
+=item SQLGetTypeInfo (superceded by get_type_info(), the DBI standard)
 
 This function maps to the ODBC SQLGetTypeInfo call.  This is a Level 1
 ODBC function.  An example of this is:
@@ -970,7 +590,7 @@ interpreted.  This "imports" the sql type names into the program's name
 space.  A very common mistake is to forget the qw(:sql_types) and
 obtain strange results.
 
-=item GetFunctions
+=item GetFunctions (now supports ODBC V3)
 
 This function maps to the ODBC API SQLGetFunctions.  This is a Level 1
 API call which returns supported driver funtions.  Depending upon how
@@ -980,11 +600,14 @@ SQL_API_ALL_FUNCTIONS (0), it will return the 100 element array.
 Otherwise, pass the number referring to the function.  (See your ODBC
 docs for help with this).
 
-=item SQLColumns
+=item SQLColumns 
 
 Support for this function has been added in version 0.17.  It looks to be
 fixed in version 0.20.
 
+Use the DBI statement handle attributes NAME, NULLABLE, TYPE, PRECISION and
+SCALE, unless you have a specific reason.
+
 =item Connect without DSN
 The ability to connect without a full DSN is introduced in version 0.21.
 
@@ -997,11 +620,15 @@ Example (using MS Access):
 
 =item SQLForeignKeys
 
+See DBI's get_foreign_keys.
+   
 =item SQLPrimaryKeys
 
+See DBI's get_primary_keys
+   
 =item SQLDataSources
 
-All handled, currently (as of 0.21)
+Handled, currently (as of 0.21), also see DBI's data_sources()
 
 =item SQLSpecialColumns
 
